#!/usr/bin/env perl

# input 1: isi alignment, e.g.,
# 0-1.1.1.1 1-1.1.1.2 3-1.1.2.1 4-1.1.2.2 5-1.1.2.r 8-1 10-1.2 11-1.2.1 13-1.2.1.2.1 13-1.2.1.2.1.1 13-1.2.1.2.1.1.r 14-1.2.1.2 15-1.3.1 15-1.3.1.r 16-1.3.2

# input 2: amr degree file, e.g.,
# j#1 /#1 join-01#1 :arg0#1.1.r p#1.1 /#1.1 person#1.1 :name#1.1.1.r p2#1.1.1 /#1.1.1 name#1.1.1 :op1#1.1.1.1.r "pierre"#1.1.1.1 :op2#1.1.1.2.r "vinken"#1.1.1.2 :age#1.1.2.r t#1.1.2 /#1.1.2 temporal-quantity#1.1.2 :quant#1.1.2.1.r 61#1.1.2.1 :unit#1.1.2.2.r y#1.1.2.2 /#1.1.2.2 year#1.1.2.2 :arg1#1.2.r b#1.2 /#1.2 board#1.2 :arg1-of#1.2.1.r h#1.2.1 /#1.2.1 have-org-role-91#1.2.1 :arg0#1.2.1.1.r p#1.2.1.1 :arg2#1.2.1.2.r d2#1.2.1.2 /#1.2.1.2 director#1.2.1.2 :mod#1.2.1.2.1.r e#1.2.1.2.1 /#1.2.1.2.1 executive#1.2.1.2.1 :polarity#1.2.1.2.1.1.r -#1.2.1.2.1.1 :time#1.3.r d#1.3 /#1.3 date-entity#1.3 :month#1.3.1.r 11#1.3.1 :day#1.3.2.r 29#1.3.2 

# input 3: amr linear file. e.g.,
# join-01 "pierre" "vinken" :age 61 year board :arg1-of have-org-role-91 director executive :polarity - :time :month 11 :day 29

# input 4: eng.origpos
# 0 1 3 4 5 7 8 10 11 12 13 14 15 16

# output
# giza alignment for the stemmed amr and eng
# 0-6 1-0 2-1 3-4 4-2 5-3 6-7 8-8 9-11 10-10 11-10 12-10 14-12 15-12 17-13

use strict;
use utf8;
use Getopt::Long;

binmode STDIN, ":utf8";
binmode STDOUT, ":utf8";
binmode STDERR, ":utf8";

my %opt;
GetOptions(\%opt, 'isi_alignment=s', 'amr_degree=s', 'amr_linear=s', 'eng_origpos=s');

open(IN_AMR_LINEAR, "<:utf8", "$opt{amr_linear}") or die $!;
open(IN_ISI, "<:utf8", "$opt{isi_alignment}") or die $!;
open(IN_ORIGPOS, "<:utf8", "$opt{eng_origpos}") or die $!;
open(IN_AMR_DEGREE, "<:utf8", "$opt{amr_degree}") or die $!;
while(my $amr_linear = <IN_AMR_LINEAR>) {
    my $isi_alignment = <IN_ISI>;
    my $origpos = <IN_ORIGPOS>;
    my $amr_degree = <IN_AMR_DEGREE>;
    chomp($amr_linear);
    chomp($isi_alignment);
    chomp($origpos);

    # key: origpos, value: stem index
    my %pos_hash; 
    &read_origpos($origpos, \%pos_hash);

    # key: amr degree, value: eng stem index (bugged)
    # key1: amr degree, key2: eng stem index
    my %isi_hash;
    &read_isi($isi_alignment, \%isi_hash, \%pos_hash);

    # foreach my $degree (sort keys %isi_hash) {
    # 	foreach my $en_index (sort keys %{$isi_hash{$degree}}) {
    # 	    print STDERR "$degree\t$en_index\n";
    # 	}
    # }

    # key 1: amr token, key 2: amr stem index
    my %amr_token_hash;
    &read_amr_linear($amr_linear, \%amr_token_hash);

    &output_giza($amr_degree, \%amr_token_hash, \%isi_hash);
}
close IN_AMR_LINEAR;
close IN_ISI;
close IN_ORIGPOS;
close IN_AMR_DEGREE;

sub read_origpos {
    my ($oripos, $pos_hash) = @_;
    my @pos_array = split(/ /, $oripos);
    for(my $stem_i = 0; $stem_i < scalar(@pos_array); $stem_i++) {
	$$pos_hash{$pos_array[$stem_i]} = $stem_i;
    }
}

sub read_isi {
    my ($isi_alignment, $isi_hash, $pos_hash) = @_;
    my @isi_array = split(/ /, $isi_alignment);
    foreach my $isi (@isi_array) {
	my ($en_index, $amr_degree) = split(/\-/, $isi);
	# $$isi_hash{$amr_degree} = $$pos_hash{$en_index}; # bug!!!
	$$isi_hash{$amr_degree}{$$pos_hash{$en_index}}++ if($$pos_hash{$en_index} ne '');
    }
}

sub read_amr_linear {
    my ($amr_linear, $amr_token_hash) = @_;
    my @amr_token_array = split(/ /, $amr_linear);
    for(my $stem_i = 0; $stem_i < scalar(@amr_token_array); $stem_i++) {
	$$amr_token_hash{$amr_token_array[$stem_i]}{$stem_i}++;
    }
}

sub output_giza {
    my ($amr_degree, $amr_token_hash, $isi_hash) = @_;
    my @token_degree_array = split(/ /, $amr_degree);
    my $giza_alignment;
    for(my $i = 0; $i < scalar(@token_degree_array); $i++) {
	my ($token, $degree) = split(/\#/, $token_degree_array[$i]);
	my ($token_next, $degree_next);
	($token_next, $degree_next) = split(/\#/, $token_degree_array[$i+1]) if($i+1 < scalar(@token_degree_array));
	next if($token eq '/' || $token_next eq '/');

	# print STDERR "==$degree==\n" if($degree eq '1.2.3.1');
	my %amr_stem_i_hash;
	my $amr_stem_i_output = '';
	if(defined $$amr_token_hash{$token}) {
	    %amr_stem_i_hash = %{$$amr_token_hash{$token}};
	}
	# アライメントが存在しなくても削除すべき
	foreach my $amr_stem_i (sort {$a <=> $b} keys %amr_stem_i_hash) {
	    # $amr_stem_i_output = $amr_stem_i;
	    # delete $$amr_token_hash{$token}{$amr_stem_i} if($$isi_hash{$degree});

	    # if($token eq 'road') {
	    if(defined $$isi_hash{$degree}) {
		# print STDERR "1:$amr_stem_i\n";
		$amr_stem_i_output = $amr_stem_i;
		# print STDERR "$amr_stem_i_output\t$degree\n";
		delete $$amr_token_hash{$token}{$amr_stem_i};

		if($amr_stem_i_output ne '') {
		    my %eng_stem_i_hash;
		    %eng_stem_i_hash = %{$$isi_hash{$degree}} if($$isi_hash{$degree});
		    foreach my $eng_stem_i (sort {$a <=> $b} keys %eng_stem_i_hash) {
			$giza_alignment .= "$amr_stem_i_output\-$eng_stem_i ";
		    }
		}

	    } else {
		# print STDERR "2:$amr_stem_i\n";
		# print STDERR scalar(keys %amr_stem_i_hash), "\n";
		delete $$amr_token_hash{$token}{$amr_stem_i} if(scalar(keys %amr_stem_i_hash) > 1);
	    }
	    # }
	    last;
	}
	# print STDERR "++$amr_stem_i_output++$degree\n" if($token eq 'road');
	# if(defined $$isi_hash{$degree} && $amr_stem_i_output ne '') {
	#     # my $eng_stem_i = $$isi_hash{$degree};
	#     # $giza_alignment .= "$amr_stem_i_output\-$eng_stem_i ";

	#     # print STDERR "$degree\n";
	#     # print STDERR "--$amr_stem_i_output--\n" if($degree eq '1.2.3.1');

	#     my %eng_stem_i_hash = %{$$isi_hash{$degree}};
	#     foreach my $eng_stem_i (sort {$a <=> $b} keys %eng_stem_i_hash) {
	# 	$giza_alignment .= "$amr_stem_i_output\-$eng_stem_i ";
	#     }
	# }
    }
    $giza_alignment =~ s/ \Z//g;
    print "$giza_alignment\n";
}
